// This template is for creating a Strategy that accesses MGannSwing

/*

### EXPORTED PLOTS ###

// WAVE PLOTS
WavePivot			# Direction of Wave (1 for up, -1 for down)
WaveTotalVolume 	# Total Wave Volume
WaveBidVolume 		# Not Valid when using MBoxPrecisionDelta
WaveAskVolume 		# Not Valid when using MBoxPrecisionDelta
WaveDeltaPercent 	# Not Valid when using MBoxPrecisionDelta
WaveDeltaVolume		# Wave Delta
WaveLength			# Wave Length (in ticks)
WaveWidth			# Wave Width (in bars)
WaveTime			# Wave Time

// ALGORITHM PLOTS
 0 = No Signal
 1 = buy, 
-1 = sell
 2 = buy in trend
-2 = sell in trend

// Explanation for buy or sell in trend on this page https://mboxwave.com/three-stooges-signal

## Single Algorithms
ThreeStoogesVolume  # Three Stooges Volume (Not used in any of the combo algorithms)
ThreeStoogesDelta	# Three Stooges Delta
Pullbacks			# Effort / Result Pullback
SPUT				# Springs / Upthrusts
BSI					# Buy / Sell Imbalance

## Combo Algorithms
BSI_PB				# Buy / Sell Imbalance + Pullback
BSI_SPUT			# Buy / Sell Imbalance + Spring / Upthrust
BSI_3SD				# Buy / Sell Imbalance + Three Stooges Delta
PB_3SD				# Pullback + Three Stooges Delta
SPUT_3SD			# Spring / Upthrust + Three Stooges Delta
DoubleAlgo			# Any 2 single algorithms combined (except Three Stooges Volume)
TripleAlgo			# Any 3 single algorithms combined (except Three Stooges Volume)



# For more detauled explanation about the Algorithms
# Watch the Semi Automated Trading Webinar here
# https://youtu.be/qWA-dNRps5U

Good Luck!
-MBox Mike

*/

namespace NinjaTrader.NinjaScript.Strategies
{
	public class MApiMGannSwingStrategy : Strategy
	{
		Indicators.MBoxWaveXE.MGannSwing myWave;

		protected override void OnStateChange()
		{
			// Initialize myWave using MGannSwing
			initMGannSwing
			(		
				// Reference to myWave
				ref myWave,
				
				// Swing Size: Enter an integer
				2,
					
				// Compute Algorithmic Signals: None, All
				AlgoSelected.All,
					
				// Algo Filter: None, Smart, TrendFollowing, Countertrend};
				AlgoFilter.Smart,
				
				// Delta Type (Choose only one, comment the rest)
				DeltaTypeEnum.MBoxPrecisionDelta
				//DeltaTypeEnum.BidAsk
				//DeltaTypeEnum.UpDownTick
				//DeltaTypeEnum.Psuedo
			);
			
			if (State == State.SetDefaults)
				Calculate = Calculate.OnEachTick;
		}

		int getConfirmedIndex(int SI)
		{
			for (int i = SI-1; i >= 0; i--)
				if (myWave.WaveConfirmed[i] != 0) return i;
			
			return 0;			
		}

		protected override void OnBarUpdate()
		{
			myWave.LegacyModeInput = false;
			
			// Only process main data series
			if (BarsInProgress != 0) return;
						
			// Wait for at least 100 bars before doing anything
			if (CurrentBar < 100) return;

			// This is required to sync with our indicator instance
			myWave.Update();
			
			// Only execute when there is an update to the Wave
			if (!(myWave.WavePivot[0] == 1 || myWave.WavePivot[0] == -1))
				return;
			
			// Get Wave Swing Index for last 3 waves
			int SI0 = 0;
			int SI1 = SI0 + (int)myWave.WaveWidth[SI0];
			int SI2 = SI1 + (int)myWave.WaveWidth[SI1];
			
			// Get Wave Direction for last 3 waves
			double WD0 = myWave.WavePivot[SI1] * -1;
			double WD1 = myWave.WavePivot[SI1];
			double WD2 = myWave.WavePivot[SI2];
			
			// Get Delta Wave Volume for last 3 waves
			double DV0 = myWave.WaveDeltaVolume[SI0];
			double DV1 = myWave.WaveDeltaVolume[SI1];
			double DV2 = myWave.WaveDeltaVolume[SI2];
			
			// Get Index when Wave 1 was confirmed (CI = Confirmed Index)
			int CI = getConfirmedIndex(SI1);			
			
			// Get Confirmed Signals from Wave 1
			double SD = myWave.ThreeStoogesDelta[CI];
			double PB = myWave.Pullbacks[CI];
			double SPUT = myWave.SPUT[CI];
			double BSI = myWave.BSI[CI];
			double BSI_PB = myWave.BSI_PB[CI];
			double BSI_SPUT = myWave.BSI_SPUT[CI];
			double BSI_3SD = myWave.BSI_3SD[CI];
			double PB_3SD = myWave.PB_3SD[CI];
			double SPUT_3SD = myWave.SPUT_3SD[CI];
			double DoubleAlgo = myWave.DoubleAlgo[CI];
			double TripleAlgo = myWave.TripleAlgo[CI];

			// NOTE ABOUT WAVE CONFIRMATION
			// WAVE 0 is always unconfirmed since it is drawn in real time			
			// WAVE 1 and up and always confirmed!
			
			// Wave 0 is always unconfirmed
			Print("###### WAVE 0 ######");
			Print("Wave Direction: "    + WD0);
			Print("WaveDeltaVolume: " + DV0);
			Print("Wave Confirmed: False");
			
			// Wave 1 is always confirmed
			Print("###### WAVE 1 ######");
			Print("Wave Direction: "    + WD1);
			Print("WaveDeltaVolume: " + DV1);
			Print("ThreeStoogesDelta: " + SD);
			Print("Pullbacks: " + PB);
			Print("SPUT: " + SPUT);
			Print("BSI: " + BSI);
			Print("BSI_PB: " + BSI_PB);
			Print("BSI_SPUT: " + BSI_SPUT);
			Print("BSI_3SD: " + BSI_3SD);
			Print("PB_3SD: " + PB_3SD);
			Print("SPUT_3SD: " + SPUT_3SD);
			Print("DoubleAlgo: " + DoubleAlgo);
			Print("TripleAlgo: " + TripleAlgo);			
			Print("Wave Confirmed: True");

			// Get Index when Wave 2 was confirmed
			CI = getConfirmedIndex(SI2);
			
			// Get Confirmed Signals from Wave 1
			SD = myWave.ThreeStoogesDelta[CI];
			PB = myWave.Pullbacks[CI];
			SPUT = myWave.SPUT[CI];
			BSI = myWave.BSI[CI];
			BSI_PB = myWave.BSI_PB[CI];
			BSI_SPUT = myWave.BSI_SPUT[CI];
			BSI_3SD = myWave.BSI_3SD[CI];
			PB_3SD = myWave.PB_3SD[CI];
			SPUT_3SD = myWave.SPUT_3SD[CI];
			DoubleAlgo = myWave.DoubleAlgo[CI];
			TripleAlgo = myWave.TripleAlgo[CI];

			// Wave 2 and greater are always confirmed
			Print("###### WAVE 2 ######");
			Print("Wave Direction: "    + WD2);
			Print("WaveDeltaVolume: " + DV2);
			Print("ThreeStoogesDelta: " + SD);
			Print("Pullbacks: " + PB);
			Print("SPUT: " + SPUT);
			Print("BSI: " + BSI);
			Print("BSI_PB: " + BSI_PB);
			Print("BSI_SPUT: " + BSI_SPUT);
			Print("BSI_3SD: " + BSI_3SD);
			Print("PB_3SD: " + PB_3SD);
			Print("SPUT_3SD: " + SPUT_3SD);
			Print("DoubleAlgo: " + DoubleAlgo);
			Print("TripleAlgo: " + TripleAlgo);			
			Print("Wave Confirmed: True");
		}
	}
}

